<?php


namespace Schaffen\Financial\Payment\Managers;


use Freelabois\LaravelQuickstart\Extendables\ManipulationManager;
use Illuminate\Support\Facades\DB;
use Schaffen\Financial\Payment\DTOs\GeneratePaymentInput;
use Schaffen\Financial\FinancialMovement\Models\Installment;
use Schaffen\Financial\Payment\Exceptions\PaymentException;
use Schaffen\Financial\Payment\Models\PaymentMethod;
use Schaffen\Financial\Payment\Repositories\PaymentRepository;
use Schaffen\Financial\Payment\Services\Discharge;
use Schaffen\Financial\Payment\Services\ManipulatePayment;

class PaymentManager extends ManipulationManager
{
    protected $validation = [
        self::CREATE => [
            "pag_data" => "optional|"
        ],
        self::EDIT => [],
        self::CREATE_MESSAGE => [],
        self::EDIT_MESSAGE => [],
    ];
    private ManipulatePayment $manipulatePayment;

    public function __construct(PaymentRepository $repository, ManipulatePayment $manipulatePayment)
    {
        parent::__construct($repository);
        $this->manipulatePayment = $manipulatePayment;
    }

    public function generatePayment(GeneratePaymentInput $input){
        DB::transaction(function () use ($input){
            /**
             * Valor da parcela somado com juros e subtraído os descontos;
             */
            $paymentValue = $this->manipulatePayment->getValue($input);
            $installment = Installment::with("financial_moviment")->find($input->getParCodigo());
            /**
             * valida se a parcela já não está paga
             */
            $this->manipulatePayment->validate($input, $installment);
            /**
             * Armazena o pagamento
             */
            $payment = $this->storeOrUpdate($input->toPaymentArray(), $input->getParCodigo() ?? null);
            /**
             * Gera a forma de pagamento DINHEIRO para este pagamento
             */
            $this->manipulatePayment->associateWithPaymentMethod(
                PaymentMethod::STANDART_PAYMENT_METHOD, $payment->pag_codigo, $input->getPagamento()
            );
            /**
             * Armazena o vínculo do pagamento com a parcela
             */
            $this->manipulatePayment->associateWithInstallment(
                $input->toAssociateWithInstallmentArray() +
                [
                    "ppg_fk_pag_codigo" => $payment->pag_codigo,
                    "ppg_valor_parcela" => $paymentValue,
                    "ppg_data_vencimento" => $installment->par_data_vencimento_atual ?? null
                ]
            );
            /**
             * Armazena os lançamentos de Conta Corrente.
             * Dividido por Juros e Multas, Adicionais e Desconto. Além do lançamento do pagamento da parcela, obviamente.
             */
            if($input->getQuitacaoRapida() == 1){//TODO:consultar parametro 516 != 2
                $ccr_codigos = Discharge::saveCurrentAccounts($input, $installment, $paymentValue);
                /**
                 * @var Installment $installment
                 * Vincula o lançamento da Conta Corrente com a Movimentação Financeira.
                 */
                $installment->financial_moviment->current_accounts()->syncWithPivotValues($ccr_codigos, [
                    "ccm_obs" => "[PAG] Pagamento de MVF: {$installment->par_fk_mvf_codigo}. Valor: {$paymentValue}",
                    "ccm_fk_pag_codigo" => $payment->pag_codigo
                ]);
            }
        });
    }
}